﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.MultiObjectiveTestFunction;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("DTLZ1", " http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ1 : MultiObjectiveTestFunction {

    private int actualSolutionSize=2;
    public override int ActualSolutionSize {
      get {
        return actualSolutionSize;
      }

      set {
        actualSolutionSize = value;
      }
    }

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[actualSolutionSize];
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {
         
        return int.MaxValue;
      }
    }

    public override int MinimumProblemSize {
      get {
        return Math.Max(2, ActualSolutionSize);
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    [StorableConstructor]
    protected DTLZ1(bool deserializing) : base(deserializing) { }
    protected DTLZ1(DTLZ1 original, Cloner cloner) : base(original, cloner) {}
    public DTLZ1() : base() {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ1(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      return Evaluate(r, actualSolutionSize);
    }

    private double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger or equal than the dimensionality of the solution(SolutionSize) ");
      }

      double[] res = new double[objectives];

      //calculate g(Xm)
      double sum = 0, length = 0;
      for (int i = objectives; i < r.Length; i++) {
        double d = r[i] - 0.5;
        sum += d * d - Math.Cos(20 * Math.PI * d);
        length += r[i] * r[i];
      }
      length = Math.Sqrt(length);
      double g = 100 * (length + sum);

      //calculating f0...fM-1
      for (int i = 0; i < objectives; i++) {
        double f = 0.5 * (i == 0 ? 1 : (1 - r[objectives - i - 1])) * (1 + g);
        for (int j = 0; j < objectives - i - 1; j++) {
          f *= r[j];
        }
        res[i] = f;
      }
      return res;
    }
  }
}
