﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.MultiObjectiveTestFunction;

namespace HeuristicLab.Problems.TestFunctions {
  [Item("Fonseca", "from // https://en.wikipedia.org/wiki/Test_functions_for_optimization [30.11.2015]")]
  [StorableClass]
  public class Fonseca : MultiObjectiveTestFunction {

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { -4, 4 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[] { false, false };
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {
        return 2;
      }
    }

    public override int MinimumProblemSize {
      get {
        return 1;
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override int ActualSolutionSize {
      get {
        return 2;
      }

      set {
      }
    }

    [StorableConstructor]
    protected Fonseca(bool deserializing) : base(deserializing) { }
    protected Fonseca(Fonseca original, Cloner cloner) : base(original, cloner) { }
    public Fonseca() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Fonseca(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      double f0 = 0.0, aux = 1.0 / Math.Sqrt(r.Length);

      //objective1
      for (int i = 0; i < r.Length; i++) {
        double d = r[i] - aux;
        f0 += d * d;
      }
      f0 = 1 - Math.Exp(-f0);

      //objective2
      double f1 = 0.0;
      for (int i = 0; i < r.Length; i++) {
        double d = r[i] + aux;
        f1 += d * d;
      }
      f1 = 1 - Math.Exp(-f1);

      double[] res = { f0, f1 };
      return res;
    }
  }
}
